##- Nanar <nanardon@mandrake.org>
##-
##- This program is free software; you can redistribute it and/or modify
##- it under the terms of the GNU General Public License as published by
##- the Free Software Foundation; either version 2, or (at your option)
##- any later version.
##-
##- This program is distributed in the hope that it will be useful,
##- but WITHOUT ANY WARRANTY; without even the implied warranty of
##- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##- GNU General Public License for more details.
##-
##- You should have received a copy of the GNU General Public License
##- along with this program; if not, write to the Free Software
##- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
# $Id: Builddistrib.pm,v 1.3 2004/08/08 17:13:43 othauvin Exp $

package Hdlist::Buildmedia::Builddistrib;

use strict;
use warnings;
use Hdlist;
use Hdlist::Buildmedia;

sub new {
    my ($class, %options) = @_;
    my $Builddistrib = {
        topdir => undef,
        infosdir => undef,
        hdlists => "hdlists",
        
        buildhdrs => 0,
        buildsynthesis => 1,
        buildhdlist => 1,
        buildmd5 => 1,
        checkrpms => 0,
       
        force => 0,
        verbosity => 4,
        
        medias => [],
    };

    foreach (keys(%$Builddistrib)) {
        defined($options{$_}) and $Builddistrib->{$_} = $options{$_};
    }
    
    bless($Builddistrib, $class)
}

sub loadconfig {
    my ($self) = @_;

    # keep compatibility with older distro
    $self->{have_media_info} = (-d "$self->{topdir}/media/media_info" || -l _) ? 
        1 :
        0;

    $self->{infosdir} ||= ($self->{have_media_info} ? 
        "$self->{topdir}/media/media_info" :
        "$self->{topdir}/Mandrake/base");
    
    open(my $h_hdlists, "<", "$self->{infosdir}/$self->{hdlists}") or die "Can't open $self->{topdir}/$self->{hdlists}";
    while (my $line = <$h_hdlists>) {
        chomp($line);
        my ($hdlist, $dir, $desc) = split(/\s+/,$line, 3);
        my $buildmedia = Hdlist::Buildmedia->new(
            topdir => $self->{topdir},
            rpmsdir => $dir,
            infosdir => $self->{have_media_info} ? "media/media_info" : "Mandrake/base",
            hdlist => $hdlist,
            synthesis => "synthesis.$hdlist",
            
            linkhdlist => $self->{have_media_info} ? [ "$dir/media_info" ] : [],
            
            buildhdrs => $self->{buildhdrs},
            buildsynthesis => $self->{buildsynthesis},
            buildhdlist => $self->{buildhdlist},
            buildmd5 => $self->{buildmd5},
            checkrpms => $self->{checkrpms},

            force => $self->{force},
            
            verbosity => $self->{verbosity},
        );
        push(@{$self->{medias}}, $buildmedia);
        
    }
    close($h_hdlists);
}

sub build {
    my ($self) = @_;
    my @reqfiles;
    foreach my $media (@{$self->{medias}}) {
        $media->stage1(
            callback => sub {
                my (%arg) = @_;
                defined($arg{header}) or do {
                    printf(STDERR "Can't read rpm %s\n", $arg{file});
                    return;
                };
                push(@reqfiles, grep { m:^/: } $arg{header}->tag(1049));
            }
        );
    }
    { my %uniq; $uniq{$_} = 1 foreach(@reqfiles); @reqfiles = keys(%uniq); }
    
    foreach my $media (@{$self->{medias}}) {
        $media->stage2(providefiles => [@reqfiles]);
    }
}
    

1;
